#include "tthread.h"
#include <QtNetwork>
#include <QtGui>
#include "parser.h"
#include <iostream>
#include "tnotify.h"
#include "singleton.h"

TThread::TThread(QObject *parent) :
    QObject(parent)
{

}

TThread::TThread(QUrl url, QObject *parent) :
    QObject(parent),
    URL(url),
    timer (new QTimer())
{
    state = new TState(TState::PENDING);
    timer->setInterval(10000);
    QObject::connect(timer, SIGNAL(timeout()), this, SLOT(checkThread()));

    QString mahurl(URL.toString());
    mahurl.chop(5);
    mahurl.append("json");
    jsonURL = QUrl(mahurl);
    QObject::connect(state, SIGNAL(isChanged()), this, SIGNAL(isChanged()));
    QNetworkAccessManager* nm = new QNetworkAccessManager();
    QNetworkReply* reply = nm->get(QNetworkRequest(jsonURL));

    QEventLoop loop;
    QObject::connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
    loop.exec();

    QString data(reply->readAll());
    if(parser::getTitleFromReply(data) != QString("")){
        name = parser::getTitleFromReply(data);
    }else{
        QWidget* whoat = new QWidget;
        QVBoxLayout* layout = new QVBoxLayout();
        whoat->setLayout(layout);
        whoat->setWindowTitle("Thread name");
        QLabel* lbl = new QLabel("We was unable to find thread's title, so put it please manualy.");
        QLineEdit* edit = new QLineEdit();
        QPushButton* ok = new QPushButton("Okay");
        layout->addWidget(lbl);
        layout->addWidget(edit);
        layout->addWidget(ok);
        whoat->show();

        QEventLoop loop2;
        QObject::connect(ok, SIGNAL(clicked()), &loop2, SLOT(quit()));
        loop2.exec();

        if (edit->text().isEmpty()){
            QMessageBox* msgBox = new QMessageBox();
            msgBox->setText("Please enter valid name for thread");
        }else{
            name = edit->text();
        }
        whoat->close();
    }
    lastReply = fetchLastPost(data);
}

TThread::TThread(QUrl url, QString Name, int State, QObject *parent) :
    QObject(parent),
    URL (url),
    name (Name),
    state (new TState(State))
{
    QObject::connect(state, SIGNAL(isChanged()), this, SIGNAL(isChanged()));
}

void TThread::setThreadNumber(QString number){
    threadNumber = number;
    emit isChanged();
}

QString TThread::getThreadNumber(void){
    return threadNumber;
}

void TThread::setThreadBoard(QString board){
    threadBoard = board;
    emit isChanged();
}

QString TThread::getThreadBoard(void){
    return threadBoard;
}

void TThread::setURL(QUrl url){
    URL = url;
    emit isChanged();
}

QUrl TThread::getURL(void){
    return URL;
}

void TThread::setState(int stateNum){
    state->setStateInt(stateNum);
}

QString TThread::getState(void){
    return state->getState();
}

QColor TThread::getColor(void){
    return state->getColor();
}

void TThread::setName(QString Name){
    name = Name;
    emit isChanged();
}

QString TThread::getName(void){
    return name;
}


QString TThread::fetchLastPost(QString data){
    QRegExp lastPostExp("\"display_id\":\\s(\\d*)");
    lastPostExp.lastIndexIn(data);
    std::cout <<"Last Post: "<< lastPostExp.cap(1).toStdString() <<std::endl;
    return lastPostExp.cap(1);
}

void TThread::startWatch(void){
    this->timer->start();
    this->setState(TState::WATCHING);
}

void TThread::checkThread(void){
    QNetworkAccessManager* nm = new QNetworkAccessManager();
    QUrl requestUrl(QString("http://dobrochan.ru/api/thread/new/").append(threadBoard).append("/").append(threadNumber).append(".json?last_post=").append(lastReply));
    QNetworkReply* reply = nm->get(QNetworkRequest(requestUrl));

    QEventLoop loop;
    QObject::connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
    loop.exec();

    QString data(reply->readAll());
    std::cout << data.toStdString() << std::endl;
    std::cout << requestUrl.toString().toStdString() << std::endl;
    QRegExp messageExp("\"message\":\\s\"([^\"]*)\"");
    int pos = 0;
    int i = 0;
    while((pos = messageExp.indexIn(data, pos)) != -1 and i < 3){
        QString message = parser::getMessageFromReply(data);
        data.replace(messageExp, "");
        TNotify* notify = &Singleton<TNotify>::Instance();
        notify->newNotify(name, message);
        i++;
        lastReply = fetchLastPost(data);
    }
}

void TThread::stopWatch(void){
    this->timer->stop();
    this->setState(TState::STOP);
    std::cout << "Thread \"" << name.toStdString() << "\" stopped." << std::endl;
}
